unit SnapObjDataset;

interface

uses
  Classes, DB, SnapBaseDataset;//, BaseDataset;

type
  TMyItem = class(TCollectionItem)
  private
    FDescrizione: string;
    procedure SetDescrizione(const Value: string);
  published
    property ID;
    property Descrizione: string read FDescrizione write SetDescrizione;
  end;

  TMyCollection = class(TCollection)
  private
    function GetItems(const Index: integer): TMyItem;
    procedure SetItems(const Index: integer; const Value: TMyItem);
  public
    property Items[const Index: integer]: TMyItem read GetItems write SetItems;
  end;

  //TSnapObjDataset = class(TGXBaseDataset)
  TSnapObjDataset = class(TSnapCustonIndexedDataset)
  private
    C: TMyCollection;
    FCurRec: Integer;
  protected
    function DoOpen: Boolean; override;
    procedure DoClose; override;
    procedure DoCreateFieldDefs; override;
    function GetCanModify: Boolean; override;
    function GetFieldValue(Field: TField): Variant; override;
    procedure SetFieldValue(Field: TField; Value: Variant); override;
    function GetRecordCount: Integer; override;
    procedure GetBlobField(Field: TField; Stream: TStream); override;
    procedure SetBlobField(Field: TField; Stream: TStream); override;
  public
    procedure AfterConstruction; override;
    procedure BeforeDestruction; override;
    property ActiveIndex;
  end;

implementation

uses
  sysutils;

{ TSnapObjDataset }

procedure TSnapObjDataset.AfterConstruction;
begin
  inherited;
  C := TMyCollection.Create(TMyItem);
end;

procedure TSnapObjDataset.BeforeDestruction;
begin
  C.Free;
  inherited;
end;

procedure TSnapObjDataset.DoCreateFieldDefs;
begin
  inherited;
  FieldDefs.Add('ID',ftInteger,0,False);
  FieldDefs.Add('DESCRIZIONE',ftString,255,False);
end;

function TSnapObjDataset.DoOpen: Boolean;
var
  i: integer;
  itm: TMyItem;
begin
  //FCurRec:=-1;
  Result := inherited DoOpen;
  for i:=0 to 20 do
  begin
    itm := C.Add as TMyItem;
    itm.Descrizione := 'Descrizione' + InttoStr(itm.ID);
  end;
//  Result := True;
end;

function TSnapObjDataset.GetFieldValue(Field: TField): Variant;
begin
  if Field.FieldName='ID' then
    Result:=C.Items[FCurrent].ID;
  if Field.FieldName='DESCRIZIONE' then
    Result:=C.Items[FCurrent].DESCRIZIONE;
end;

function TSnapObjDataset.GetRecordCount: Integer;
begin
  Result := C.Count;
end;

procedure TSnapObjDataset.SetFieldValue(Field: TField; Value: Variant);
begin
  if Field.FieldName='DESCRIZIONE' then
    C.Items[FCurrent].DESCRIZIONE := Value;
end;

procedure TSnapObjDataset.DoClose;
begin
  inherited;
end;

procedure TSnapObjDataset.GetBlobField(Field: TField; Stream: TStream);
begin
  inherited;
end;

procedure TSnapObjDataset.SetBlobField(Field: TField; Stream: TStream);
begin
  inherited;
end;

function TSnapObjDataset.GetCanModify: Boolean;
begin
  Result := True;
end;

{ TMyItem }

procedure TMyItem.SetDescrizione(const Value: string);
begin
  FDescrizione := Value;
end;

{ TMyCollection }

function TMyCollection.GetItems(const Index: integer): TMyItem;
begin
  Result := TMyItem(inherited Items[Index]);
end;

procedure TMyCollection.SetItems(const Index: integer;
  const Value: TMyItem);
begin
  inherited Items[Index] := Value;
end;


end.
